<?php

namespace Drupal\migrate_wizard\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\migrate_wizard\MWConfigBuilder;
use Drupal\migrate_wizard\MWManageDataService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure migrate_wizard.
 */
class OriginVocabularyFields extends BaseFormFields {

  use BaseFormFieldsTrait;
  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'manage_origin_vocabulary_fields';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, string $origin_bundle = NULL) {
    $this->originType = 'vocabulary';
    $this->fieldsOriginBundle = $this->manageFormService->getFieldsOriginBundle($this->originBundle, $this->originType);
    $this->manageFormService->setFieldsOriginBundle($this->fieldsOriginBundle);
    $type_fields_checkboxes = [
      'preserve_tid' => 'Preserve tid',
      'import_users' => 'Import users',
      'import_path_alias' => 'Import path alias',
    ];
    $this->checkboxes = array_merge($this->checkboxes, $type_fields_checkboxes);
    $form['origin_type'] = [
      '#type' => 'hidden',
      '#value' => 'vocabulary',
    ];

    $this->langCodes['default_lang']['value'] = $this->mwManageDataService->getOriginDefaultLangcode();

    $config_translation = $this->mwManageDataService->getOriginLanguages($form["origin_type"]["#value"], $origin_bundle);

    foreach($config_translation["origin_languages"] as $language){
      $this->langCodes[$language.'_lang'] = [
        'markup' => $language,
        'value' => $language,
      ];
    }
    if(count($config_translation["origin_languages"]) > 0){
      $this->i18nMode = $config_translation['i18n_mode'];
    }
    return parent::buildForm($form, $form_state, $origin_bundle);
  }
  public function getTitle($type = null): \Drupal\Core\StringTranslation\TranslatableMarkup
  {
    return parent::getTitle('vocabulary');
  }
}
