<?php

namespace Drupal\migrate_wizard\Plugin\FieldTypeMW;

use Drupal\migrate_wizard\FieldTypeMWBase;

/**
 * Provides 'default_taxonomy_term' field type.
 *
 * @FieldType(
 *   id = "default_taxonomy_term",
 * )
 */
class FieldTypeMWDefaultTaxonomyTerm extends FieldTypeMWBase {

  public static function getMigrationConfig(&$wildcard_yml_config, $array_config, $origin_node, $langcode = NULL, $translation = FALSE) {

    $suffix = '';
    if($translation){
      $suffix = '_'.$langcode;
    }
    $wildcard_yml_config['langcode'] = $langcode;
    $wildcard_yml_config['id'] = $origin_node.$suffix;
    $wildcard_yml_config['label'] = $origin_node.$suffix;
    $wildcard_yml_config['migration_group'] = getenv('MIGRATE_DB_NAME');
    $wildcard_yml_config['migration_tags'][0] = getenv('MIGRATE_DB_NAME');
    $wildcard_yml_config['source']['bundle'] = $origin_node.$suffix;
    $wildcard_yml_config['source']['origin_type'] = $array_config['origin_type'];
    $wildcard_yml_config["process"]["vid"][0]["default_value"] = array_key_first($array_config);
    $wildcard_yml_config['process']['parent_id'][1]['migration'] = $origin_node.$suffix;
    $wildcard_yml_config['destination']['plugin'] = 'entity:taxonomy_term';
    $wildcard_yml_config['source']['constants']['langcode'] = $langcode;
    if (isset($array_config['preserve_tid']) && $array_config['preserve_tid']) {
      $wildcard_yml_config['process']['tid'] = 'tid';
    }
    if($translation){
      switch($array_config["i18n_mode"]){
        case 1:
          $wildcard_yml_config['process']['tid'] = [
            'plugin' => 'migration_lookup',
            'migration' => $origin_node,
            'source' => 'tid',
            'no_stub' => TRUE,
          ];
          break;
        case 4:
          $wildcard_yml_config['process']['tid'] = [
            'plugin' => 'migration_lookup',
            'migration' => $origin_node,
            'source' => 'ttd_2_tid',
            'no_stub' => TRUE,
          ];
          break;
      }
      $wildcard_yml_config['source']['translations'] = TRUE;
      $wildcard_yml_config['source']['constants']['translations'] = TRUE;

      $wildcard_yml_config['process']['langcode'] = [
        'plugin' => 'default_value',
        'default_value' => $langcode
      ];
      $wildcard_yml_config['destination']['default_bundle'] = $origin_node;
      $wildcard_yml_config['destination']['translations'] = TRUE;
      $wildcard_yml_config['destination']['langcode'] = $langcode;
      $wildcard_yml_config['langcode'] = $langcode;
    }
    $wildcard_yml_config['process']['type'] = [
      'plugin' => 'default_value',
      'default_value' => array_key_first($array_config),
    ];
    if ($array_config["origin_type"] == 'node') {
      $wildcard_yml_config["process"]["weight"][0]["source"] = 'nid';
    }
    if ($array_config['import_path_alias']) {
      $wildcard_yml_config['process']['path'][] = [
        'plugin' => 'skip_on_empty',
        'source' => 'alias',
        'method' => 'process',
      ];
    }
  }
}
