<?php

namespace Drupal\migrate_wizard\Plugin\FieldTypeMW;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\migrate_wizard\FieldTypeMWBase;
use Drupal\migrate_wizard\FieldTypeMWInterface;

/**
 * Provides 'field_collection' field type.
 *
 * @FieldType(
 *   id = "field_collection",
 * )
 */
class FieldTypeMWFieldCollection extends FieldTypeMWBase {

  public static function getMigrationConfig(&$wildcard_yml_config, $origin_field, $data_field, $type_source, $config, $entityFieldManager, $fieldTypeMWManager, $field_collection = NULL) {

    $destiny_explode = explode('/', $data_field['destiny']);
    $array_config[$destiny_explode[0]][] = [
      'plugin' => 'skip_on_empty',
      'method' => 'process',
      'source' => $origin_field . '_value',
    ];
    $array_config[$destiny_explode[0]][] = [
      'plugin' => 'create_paragraph_entity',
      'source' => [],
      'bundle' => $data_field["target_bundles"],
    ];
    $array_config[$destiny_explode[0]][] = [
      'plugin' => 'sub_process',
      'process' => ['target_id' => '0', 'target_revision_id' => '1'],
    ];
    $array_config[$destiny_explode[0]][] = [
      'plugin' => 'multiple_values',
    ];

    if(!$field_collection){
      $wildcard_yml_config['process'] = array_merge($wildcard_yml_config['process'], $array_config);
    }else{
      foreach($array_config[$destiny_explode[0]] as $value){
        $wildcard_yml_config['process'] = parent::insertBeforeKey($wildcard_yml_config['process'], $field_collection, $destiny_explode[0], $value);
      }
    }

    foreach($data_field["children"] as $sub_field_name => $sub_data_field){
      if ($data_field['destiny'] != 'none') {
        $field_type_mw = $fieldTypeMWManager->createReflectionClass($sub_data_field['type']);
        $sub_data_field_explode = explode('/', $sub_data_field['destiny']);
        $sub_data_field['destiny'] = 'pseudo_'.$sub_data_field_explode[0];

        $field_type_mw->name::getMigrationConfig($wildcard_yml_config, $sub_field_name, $sub_data_field, $type_source, $config, $entityFieldManager, $fieldTypeMWManager, $destiny_explode[0]);
      }
    }

    $base_fields = $entityFieldManager->getBaseFieldDefinitions('paragraph');
    $full_fields = $entityFieldManager->getFieldDefinitions('paragraph', $data_field["target_bundles"]);
    $extra_fields = array_diff_key($full_fields, $base_fields);

    foreach($extra_fields as $key_extra_fields => $extra_field){
      $wildcard_yml_config['process'][$destiny_explode[0]][1]['source'][] = '@pseudo_' . $key_extra_fields;
    }
  }

  public function addQueryByType($alias, $table, $data_field, $wildcard, $translation, $field_collection = NULL) {

    $base_field = str_replace('field_data_', '', $table).'_value';
    if($field_collection == NULL){
      $wildcard->query->leftJoin($table, $alias, "{$wildcard->getValue('leftJoinTable')} = $alias.entity_id AND $alias.entity_type = '{$wildcard->getValue('entityType')}' AND $alias.bundle = '{$wildcard->getValue('bundle')}'");
    }else{
      $wildcard->query->leftJoin($table, $alias, "{$field_collection['left_join_table']} = $alias.entity_id AND $alias.entity_type = 'field_collection_item' AND $alias.bundle = '{$field_collection['bundle']}'");
    }
    $wildcard->query->addExpression("GROUP_CONCAT(DISTINCT CONCAT (COALESCE($alias.{$base_field}, '')) ORDER BY $alias.delta ASC SEPARATOR '||')", $base_field , []);

    foreach($data_field["children"] as $sub_field_name => $sub_data_field){
       if ($sub_data_field['type'] != 'image' && $sub_data_field['type'] != 'file' && $sub_data_field["destiny"] != 'none' && !array_key_exists($sub_field_name, $wildcard->query->getTables())) {
         $field_collection = ['left_join_table' => $alias.'.'.$base_field, 'bundle' => $alias];
         $wildcard->addQuery($sub_field_name, $sub_data_field, $translation, $field_collection);
       }
    }
  }
}
