<?php

namespace Drupal\migrate_wizard\Plugin\FieldTypeMW;

use Drupal\migrate_wizard\FieldTypeMWBase;

/**
 * Provides 'generic_file' field type.
 *
 * @FieldType(
 *   id = "generic_file",
 * )
 */
class FieldTypeMWGenericFile extends FieldTypeMWBase {

  public static function getMigrationConfig(&$wildcard_yml_config, $origin_field, $data_field, $type_source, $config, $entityFieldManager, $fieldTypeMWManager, $field_collection = NULL) {

    $fields_content_type_destiny = $entityFieldManager->getFieldDefinitions($type_source, $wildcard_yml_config["process"]["type"]["default_value"]);
    $settings_field = $fields_content_type_destiny[$data_field['destiny']]->getSettings();
    $array_config = [];
    if(count($settings_field['handler_settings']) > 0){
      $bundle = key($settings_field['handler_settings']['target_bundles']);
      if(!isset($wildcard_yml_config['process'][$data_field['destiny']])){
        if($data_field['type'] == 'image'){
          $array_config['pseudo_'.$origin_field][] = [
            'plugin' => 'get_file',
            'source' => [$origin_field, $origin_field.'_alt', $origin_field.'_title'],
          ];
        }elseif($data_field['type'] == 'file'){
          $array_config['pseudo_'.$origin_field][] = [
            'plugin' => 'get_file',
            'source' => [$origin_field, $origin_field.'_description'],
          ];
        }
        $array_config['pseudo_'.$origin_field][] = [
          'plugin' => 'create_media',
          'entity_type' => 'media',
          'bundle' => $bundle,
          'field_name' => $data_field["media_field"],
        ];
        $array_config['pseudo_'.$data_field['destiny']][] = [
            'plugin' => 'media_concat_data',
            'source' => ['@pseudo_'.$origin_field],
        ];
        $array_config[$data_field['destiny']][] = [
          'plugin' => 'sub_process',
          'source' => '@pseudo_'.$data_field['destiny'],
          'method' => 'process',
          'process' => ['target_id' => 'mid'],
        ];
      }else{
        $insert_value = [];
        if($data_field['type'] == 'image'){
          $insert_value = [
            'plugin' => 'get_file',
            'source' => [$origin_field, $origin_field.'_alt', $origin_field.'_title'],
          ];
        }elseif($data_field['type'] == 'file'){
          $insert_value = [
            'plugin' => 'get_file',
            'source' => [$origin_field, $origin_field.'_description'],
          ];
        }
        $wildcard_yml_config['process'] = parent::InsertBeforeKey($wildcard_yml_config['process'], 'pseudo_'.$data_field['destiny'], 'pseudo_'.$origin_field, $insert_value);
        $insert_value = [
          'plugin' => 'create_media',
          'entity_type' => 'media',
          'bundle' => $bundle,
          'field_name' => $data_field["media_field"],
        ];
        $wildcard_yml_config['process'] = parent::InsertBeforeKey($wildcard_yml_config['process'], 'pseudo_'.$data_field['destiny'], 'pseudo_'.$origin_field, $insert_value);
        $wildcard_yml_config['process']['pseudo_'.$data_field['destiny']][0]['source'][] = '@pseudo_'.$origin_field;
      }
    }else{
      $array_config[$data_field['destiny']][] = [
        'plugin' => 'skip_on_empty',
        'source' => $origin_field,
        'method' => 'process',
      ];

      if ($data_field['type'] == 'image') {
        $array_config[$data_field['destiny']][] = [
          'plugin' => 'get_file',
          'field_name' => $data_field["media_field"],
          'source' => [
            $origin_field,
            $origin_field . '',
            $origin_field . '_title',
          ],
        ];
        $array_config[$data_field['destiny']][] = [
          'plugin' => 'sub_process',
          'method' => 'process',
          'process' => [
            'target_id' => 'target_id',
            'alt' => 'alt',
            'title' => 'title'
          ],
        ];
      }
      elseif ($data_field['type'] == 'file') {
        $array_config[$data_field['destiny']][] = [
          'plugin' => 'get_file',
          'source' => [$origin_field, $origin_field . '_description'],
        ];
        $array_config[$data_field['destiny']][] = [
          'plugin' => 'sub_process',
          'method' => 'process',
          'process' => [
            'target_id' => 'target_id',
            'description' => 'description',
          ],
        ];
      }
    }

    if(!$field_collection){
      $wildcard_yml_config['process'] = array_merge($wildcard_yml_config['process'], $array_config);
    }else{
      foreach($array_config[$data_field['destiny']] as $value){
        $wildcard_yml_config['process'] = parent::insertBeforeKey($wildcard_yml_config['process'], $field_collection, $data_field['destiny'], $value);
      }
    }
  }

  public function addQueryByType($alias, $table, $data_field, $wildcard, $translation, $field_collection = NULL) {

    $base_field = str_replace('field_data_', '', $table);
    $talias = $alias . '_file';

    if($field_collection == NULL){
      $wildcard->query->leftJoin($table, $alias, "{$wildcard->getValue('leftJoinTable')} = $alias.entity_id AND $alias.entity_type = '{$wildcard->getValue('entityType')}' AND $alias.bundle = '{$wildcard->getValue('bundle')}'");
    }else{
      $wildcard->query->leftJoin($table, $alias, "{$field_collection['left_join_table']} = $alias.entity_id AND $alias.entity_type = 'field_collection_item' AND $alias.bundle = '{$field_collection['bundle']}'");
    }

    $wildcard->query->leftJoin('file_managed', $talias, "$alias.{$base_field}_fid = $talias.fid");
    $wildcard->query->addExpression("GROUP_CONCAT(DISTINCT COALESCE($talias.uri, '') ORDER BY $alias.delta ASC  SEPARATOR '||')", $base_field, []);

    switch($data_field["type"]){
      case 'image':
        $wildcard->query->addExpression("GROUP_CONCAT(DISTINCT CONCAT (COALESCE($alias.{$base_field}_alt, '')) ORDER BY $alias.delta ASC SEPARATOR '||')", $base_field.'_alt', []);
        $wildcard->query->addExpression("GROUP_CONCAT(DISTINCT CONCAT (COALESCE($alias.{$base_field}_title, '')) ORDER BY $alias.delta ASC SEPARATOR '||')", $base_field.'_title', []);
      break;
      case'file':
        $wildcard->query->addExpression("GROUP_CONCAT(DISTINCT COALESCE($alias.{$base_field}_description, '') ORDER BY $alias.delta ASC  SEPARATOR '||')", $base_field.'_description', []);
      break;
    }
  }
}
