<?php

namespace Drupal\migrate_wizard\Plugin\migrate\process;

use Drupal\media\Entity\Media;
use Drupal\migrate\ProcessPluginBase;
use Drupal\migrate\MigrateExecutableInterface;
use Drupal\migrate\Row;

/**
 * Create media.
 *
 * @MigrateProcessPlugin(
 *   id = "create_media"
 * )
 */
class CreateMedia extends ProcessPluginBase {

  /**
   * {@inheritdoc}
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  protected $migrationUrlFilesProd;

  /**
   * {@inheritdoc}
   */
  protected $fileSystem;

  /**
   * {@inheritdoc}
   */
  protected $entityManager;

  /**
   * {@inheritdoc}
   */
  public function transform($value, MigrateExecutableInterface $migrate_executable, Row $row, $destination_property) {
    if($value == null){
      return;
    }
    $this->config = \Drupal::configFactory()->getEditable('migrate_wizard.settings');
    $this->migrationUrlFilesProd = $this->config->get('migrate_wizard_settings')['migration_url_files_prod'];
    $this->fileSystem = \Drupal::service('file_system');
    $this->entityManager = \Drupal::service('entity_type.manager');

    $bundle = $this->configuration['bundle'] ?? '';
    $field_name = $this->configuration['field_name'] ?? '';
    $new_values = [];

    foreach ($value as $val) {
      if(is_array($val)){
        $new_values[] = ['mid' => $this->createMediaFromFileId($val, $bundle, $field_name)];
      }
    }
    return $new_values;
  }

  /**
   * Create media from file id.
   */
  protected function createMediaFromFileId(array $file_data, string $bundle, string $field_name, $fields = []) {

    $file_description = '';
    $file_alt = '';
    $file_title = '';
    $fid = $file_data["target_id"];
    $file = $this->entityManager->getStorage('file')->load($fid);

    if (count($file_data) == 2) {
      $file_description = $file_data['description'] != '' ? $file_data['description'] : $file->getFilename();
    }
    elseif (count($file_data) == 3) {
      $file_alt = $file_data['alt'] != '' ? $file_data['alt'] : $file->getFilename();
      $file_title = $file_data['title'] != '' ? $file_data['title'] : $file->getFilename();
    }

    // loadByProperties media by fid.
    $medias = $this->entityManager->getStorage('media')->loadByProperties([$field_name => $fid]);
    $media = reset($medias) ?: NULL;

    if ($media == NULL) {
      $media = Media::create([
        'bundle' => $bundle,
        'uid' => 1,
        'name' => count($file_data) == 2 ? $file_description : $file_title,
        $field_name => [
          'target_id' => $fid,
          'display' => 1,
          'alt' => count($file_data) == 2 ? $file_description : $file_alt,
        ],
      ]);
    }
    else {
      $media->set('name', count($file_data) == 2 ? $file_description : $file_title);
      $media->set($field_name, [
        'target_id' => $fid,
        'display' => 1,
        'alt' => count($file_data) == 2 ? $file_description : $file_alt,
      ]);
    }

    foreach ($fields as $key => $value) {
      $media->set($key, $value);
    }

    $media->save();

    return $media->id();
  }

}
