<?php

namespace Drupal\migrate_wizard\Plugin\migrate\process;

use Drupal\file\Entity\File;
use Drupal\migrate\MigrateSkipProcessException;
use Drupal\migrate\ProcessPluginBase;
use Drupal\migrate\MigrateExecutableInterface;
use Drupal\migrate\Row;
use Drupal\Core\File\FileSystemInterface;

/**
 * Get file.
 *
 * @MigrateProcessPlugin(
 *   id = "get_file"
 * )
 */
class GetFile extends ProcessPluginBase {

  /**
   * {@inheritdoc}
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  protected $migrationUrlFilesProd;

  /**
   * {@inheritdoc}
   */
  protected $fileSystem;

  /**
   * {@inheritdoc}
   */
  protected $entityManager;

  /**
   * {@inheritdoc}
   */
  public function transform($value, MigrateExecutableInterface $migrate_executable, Row $row, $destination_property) {
    if($value[0] == null){
      return;
    }
    $this->config = \Drupal::configFactory()->getEditable('migrate_wizard.settings');
    $this->migrationUrlFilesProd = $this->config->get('migrate_wizard_settings')['migration_url_files_prod'];
    $this->fileSystem = \Drupal::service('file_system');
    $this->entityManager = \Drupal::service('entity_type.manager');

    $tmp_values = [];
    $new_values = [];
    if (count($value) == 2) {
      $array_urls = explode('||', $value[0]);
      $array_descriptions = explode('||', $value[1]);
      foreach ($array_urls as $key => $url) {
        $tmp_values[] = [
          'url' => $url,
          'description' => $array_descriptions[$key] ?? '',
        ];
      }
      foreach ($tmp_values as $tmp_value) {
        $new_values[] = $this->createFile($tmp_value, $row);
      }
    }
    elseif (count($value) == 3) {
      $array_urls = explode('||', $value[0]);
      $array_alts = explode('||', $value[1]);
      $array_titles = explode('||', $value[2]);
      foreach ($array_urls as $key => $url) {
        $tmp_values[] = [
          'url' => $url,
          'alt' => $array_alts[$key] ?? '',
          'title' => $array_titles[$key] ?? '',
        ];
      }
      foreach ($tmp_values as $tmp_value) {
        $new_values[] = $this->createFile($tmp_value, $row);
      }
    }
    return $new_values;
  }

  /**
   * Create file.
   */
  protected function createFile(array $file_data, $row) {
    $file_url = '';
    $file_description = '';
    $file_alt = '';
    $file_title = '';

    if (count($file_data) == 2) {
      $file_url = str_replace('public://', $this->migrationUrlFilesProd, $file_data['url']);
      $file_description = $file_data['description'] != '' ? $file_data['description'] : basename($file_url);
    }
    elseif (count($file_data) == 3) {
      $file_url = str_replace('public://', $this->migrationUrlFilesProd, $file_data['url']);
      $file_alt = $file_data['alt'] != '' ? $file_data['alt'] : basename($file_url);
      $file_title = $file_data['title'] != '' ? $file_data['title'] : basename($file_url);
    }

    if (count($file_data) == 2) {
      $file_url = str_replace('public://', $this->migrationUrlFilesProd, $file_data['url']);
    }
    elseif (count($file_data) == 3) {
      $file_url = str_replace('public://', $this->migrationUrlFilesProd, $file_data['url']);
    }

    $directory = str_replace($this->migrationUrlFilesProd, '', $file_url);
    if (basename($file_url) != $directory) {
      $directory = str_replace(basename($file_url), '', $directory);
      $directory = 'public://' . $directory;
    }
    else {
      $directory = 'public://';
    }

    $uri = $directory . basename($file_url);

    $this->fileSystem->prepareDirectory($directory, FileSystemInterface:: CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS);

    if (file_exists($directory . '/' . basename($file_url))) {

      $files = $this->entityManager->getStorage('file')->loadByProperties(['uri' => $uri]);

      $file = reset($files) ?: NULL;
      if ($file == NULL) {
        // Create file with File::create.
        $file = File::create([
          'uri' => $uri,
          'uid' => 1,
          'status' => 1,
        ]);
        $file->save();
      }
    }
    else {

      $url_downloadable = get_headers($file_url);

      // Check the response code.
      if (strpos($url_downloadable[0], '200') !== FALSE) {
        $file = system_retrieve_file($file_url, $directory, TRUE, FileSystemInterface::EXISTS_REPLACE);
      }
      else {
        \Drupal::logger('migrate_wizard')->notice(' File not found: ' . $file_url.' in node: '.$row->getSourceProperty('nid'));
        throw new MigrateSkipProcessException($file_url . ' Error ' . $url_downloadable[0]);
      }

    }

    if ($file instanceof \Drupal\file\FileInterface) {
      if (count($file_data) == 2) {
        $value_file = [
          'target_id' => $file->id(),
          'description' => $file_description,
        ];
      }
      elseif (count($file_data) == 3) {
        $value_file = [
          'target_id' => $file->id(),
          'alt' => $file_alt,
          'title' => $file_title,
        ];
      }
      return $value_file;
    }else{
      \Drupal::logger('migrate_wizard')->notice(' File not found: ' . $file_url.' in node: '.$row->getSourceProperty('nid'));
    }
  }

}
